# Copyright 2001, 2002 SolidCode BV (http://www.solidcode.net)

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package pdfGraphics;
use Subpath;
use Gfxstate;

use strict;

# colorspaces and vars
my $ds_stroke = '';		# Devices Space for (Non) Stroking operations
my $ds_nonstroke  = '';
my @strokecolor = (1,1,1,1);
my @nonstrokecolor = (1,1,1,1);
my $font_type = '';

my @curp = (0,0); # current point
my $gfxstate = Gfxstate->new;
my $subpath = Subpath->new;
my @stack = [];

#my $ymarge = 1600;
my @mediabox = ();
my $rotate = 0;
my $pagerotate = 0;
# --------------------------------------------------------------------------------------

sub storeState {
	push @stack, $gfxstate->deepcopy;
	$gfxstate->destroy;
}

sub restoreState {
	$gfxstate->destroy;
	if (@stack <= 0) {
		$gfxstate->new;
	}
	else {
		$gfxstate = ( pop @stack )->deepcopy;
		setPoint( $gfxstate->startpoint );
	}
}

# --------------------------------------------------------------------------------------

sub setStrokingCS {
	$ds_stroke = $_[0];
}

sub setNonStrokingCS {
	$ds_nonstroke = $_[0];
}

sub setStrokingColor {
	@strokecolor[0..3] = @_[0..3];
}
sub setNonStrokingColor {
	@nonstrokecolor[0..3] = @_[0..3];
}

sub convertCMYKtoRGB {
	my @cmyk = (0,0,0,0);
	my @rgbcolor = (0,0,0);
	for (my $i=0; $i<4; $i++) {
		$cmyk[$i] = ($_[$i]?$_[$i]:0);
	}
	$rgbcolor[0] = 1 - (($cmyk[0]+$cmyk[3] < 1)?($cmyk[0]+$cmyk[3]):1);
	$rgbcolor[1] = 1 - (($cmyk[1]+$cmyk[3] < 1)?($cmyk[1]+$cmyk[3]):1);
	$rgbcolor[2] = 1 - (($cmyk[2]+$cmyk[3] < 1)?($cmyk[2]+$cmyk[3]):1);
	return @rgbcolor;
}

sub strokeColor {
	if ($ds_stroke eq 'DeviceGray') {
		my $gc = ($strokecolor[0]*255);
		return sprintf("#%.2x%.2x%.2x", $gc,$gc,$gc);
	} elsif ($ds_stroke eq 'DeviceRGB') {
		return sprintf("#%.2x%.2x%.2x", @strokecolor[0..2]);
	} elsif ($ds_stroke eq 'DeviceCMYK') {
		# still gotta convert CMYK to RGB....
		return sprintf("#%.2x%.2x%.2x", convertCMYKtoRGB(@strokecolor));
	} else {
		#	return "none"
		return "#0000aa";
	}
}

sub fillColor {
	if ($ds_nonstroke eq 'DeviceGray') {
		my $gc = ($nonstrokecolor[0]*255);
		return sprintf("#%.2x%.2x%.2x", $gc,$gc,$gc);
	} elsif ($ds_nonstroke eq 'DeviceRGB') {
		return sprintf("#%.2x%.2x%.2x", @nonstrokecolor[0..2]);
	} elsif ($ds_nonstroke eq 'DeviceCMYK') {
		# first convert CMYK to RGB....
		return sprintf("#%.2x%.2x%.2x", convertCMYKtoRGB(@nonstrokecolor));
	} else {
		#	return "none"
		return "#0000aa";
	}
}

# --------------------------------------------------------------------------------------

sub setMediaBox {
	(@mediabox) = (@_);    
	Subpath::setYmarge ( $mediabox[3] ); #(- mb[1] ? )
}

sub rotateDiff {
	return (shift(@_) + $pagerotate);
}

sub setPageRotate {
	$pagerotate = shift(@_);
}

sub setRotate {
	$rotate = shift(@_);
}

sub setLineWidth {
	$gfxstate->linewidth($_[0]);
}

sub setDashPattern {
	$gfxstate->dashpattern($_[0]);
}

# ---- PATH operators -----------------------------------------------------------

sub setPoint {
	# set cursor 
	@curp = @_[0,1];
}

sub clearPath {
	$subpath->destroy;
}

sub clearPathList {
	$gfxstate->clearpathlist;
}

sub addPath {
	my $type = shift;
	$subpath->set($type, @_);
	$gfxstate->push($subpath);
	setPoint(@_[-2,-1]);
}

sub addDummy {
	addPath('empty', @curp, @_);
}

sub addLine {
	addPath('line', @curp, @_);
}

sub addRectangle {
	addPath('rect', $_[0], $_[1], @_[2,3]);
}

sub addCurve {
	# current point?
	addPath('curve', @_);
}

sub addCurveB {
	addPath('curve', @curp, @_);
}

sub addCurveC {
	my (@coords) = @_;
	addPath('curve', @coords[0,1], @curp, @coords[2,3]);
}

sub closePathList {
	if (! ($gfxstate->isEmpty) || ($gfxstate->isClosed) ) {
		my @startpoint = $gfxstate->startpoint;
		addPath('line', @curp, @startpoint);
	}
}

sub closePathListEmpty {
	if (! ($gfxstate->isEmpty) || ($gfxstate->isClosed) ) {
		my @startpoint = $gfxstate->startpoint;
		addPath('empty', @curp, @startpoint);
	}
}

sub strokePathList {
	$gfxstate->stroke(strokeColor());
}

sub fillPathList {
	# $gfxstate->stroke(fillColor());
	$gfxstate->fill(fillColor());
}

sub paintPathList {
	$gfxstate->fill('#a09050');
}


# ------ TEXT ------- #
sub setFontType {
	$font_type = $_[0];
}

sub getFontType {
	return ($font_type)?$font_type:'';
}

sub printText {
	# printText (x,y, size,width,style, text)
	setPoint(@_[0,1]);
	my $ftype 	  = getFontType();
	my $fsize 	  = $_[2];
	my $textwidth = $_[3];
	my $extrastyle= $_[4];
	my $tekst 	  = $_[5];

	if (length($tekst) > 0) {
		printf("\t<text x=\"%.2f\" y=\"%.2f\" class=\"%s\" width=\"%.2f\" style=\"font-size:%.2f; font-color:%s; font-family: %s, Times New Roman, serif; %s\">%s</text>\n", $curp[0], Subpath::flipY($curp[1]), $ftype, $textwidth, $fsize, fillColor(), $ftype, $extrastyle, $tekst);
	}
 }

# -- for testing purposes -- #
sub highLite {
	printf ("\t<rect class=\"dummy\" x=\"%.2f\" y=\"%.2f\" width=\"%.2f\" height=\"%.2f\" style=\"fill:%s; stroke: #CCAA80; stroke-width: %.2f\"/>\n", $curp[0], ($curp[1])-4, $_[0], -6, "#FFDcA8", $_[1]);
}

1;
